; CIO values and macros.
; License: GPL-2, see LICENSE
; This version (c) 2022 Mark Ormston <memso@memso.net>
; Original copyright (c) 2005 Matthias Reichl

.IFNDEF IncludeFile_CIO
IncludeFile_CIO = 1
.INCLUDE "AtariOS.inc"

; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; Constants

; Word, The maximum length to support for TXT commands, passed through AtariOS::ICBL / AtariOS::ICBL + 1.
ICBL_TXT_MAXLENGTH  = 127



; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; Macros

; *************************************************************************************************
; Sets the channel number to use in X so that all IOCBs can be accessed in "LDA AtariOS::ICCOM, X" format.
; Parameters:
;   channel         Byte, The channel to use. 0 to 7 permitted.
.MACRO SetCIOChannelOffset channel
	LDX #channel * 16
.ENDMACRO



; *************************************************************************************************
; Opens a DOS-style file by name.
; Parameters:
;   channel         Byte, The channel to use.
;   aux1            Byte, Auxiliary value 1 to pass to the channel's AtariOS::ICAX1.
;   aux2            Byte, Auxiliary value 2 to pass to the channel's AtariOS::ICAX2.
;   filename        Address, The address of the file name to use.
.MACRO OPEN channel, aux1, aux2, filename
	; Prepare X
	SetCIOChannelOffset channel

	; Pass the auxiliary values
	LDA #aux1
	STA AtariOS::ICAX1,X
	LDA #aux2
	STA AtariOS::ICAX2,X

	; Set up opening the file
	LDA #ATARIOS_ICCOM::Open
	STA AtariOS::ICCOM,X
	LDA #.LOBYTE(filename)
	STA AtariOS::ICBA,X
	LDA #.HIBYTE(filename)
	STA AtariOS::ICBA + 1,X

	; Call the CIO vector
	JSR AtariOS::CIOV
.ENDMACRO



; *************************************************************************************************
; Closes a file opened on an IOCB channel.
; Parameters:
;   channel         Byte, The channel to use.
.MACRO CLOSE channel
	; Prepare X
	SetCIOChannelOffset channel

	; Close the file
	LDA #ATARIOS_ICCOM::Close
	STA AtariOS::ICCOM,X
	JSR AtariOS::CIOV
.ENDMACRO



; *************************************************************************************************
; ??? - I think this is to get the current sector that CIO is working with.
; Parameters:
;   channel         Byte, The channel to use.
;   sector          Address, The address to a word to store the sector # to.
.MACRO NOTE channel, sector
	; Prepare X
	SetCIOChannelOffset channel

	; Call AtariOS::CIOV with the Note command
	LDA #ATARIOS_ICCOM::Note
	STA AtariOS::ICCOM,X
	JSR AtariOS::CIOV

	; Get the sector
	LDA AtariOS::ICAX3,X
	STA sector
	LDA AtariOS::ICAX4,X
	STA sector + 1
.ENDMACRO



; *************************************************************************************************
; ??? - Does this actually print to a printer?
; Parameters:
;   channel         Byte, The channel to use.
;   label           Address, The address of the text. No idea on the format.
.MACRO PRINT channel, label
	; Prepare X
	SetCIOChannelOffset channel

	LDA #ATARIOS_ICCOM::PutText
	STA AtariOS::ICCOM,X
	LDA #.LOBYTE(label)
	STA AtariOS::ICBA,X
	LDA #.HIBYTE(label)
	STA AtariOS::ICBA + 1,X
	LDA #.LOBYTE(ICBL_TXT_MAXLENGTH)
	STA AtariOS::ICBL,X
	LDA #.HIBYTE(ICBL_TXT_MAXLENGTH)
	STA AtariOS::ICBL + 1,X
	JSR AtariOS::CIOV
.ENDMACRO



; *************************************************************************************************
; ??? - Does this actually print to a printer?
; Parameters:
;   channel         Byte, The channel to use.
;   pointer         Address, The address of a pointer to the text to use. No idea on the format.
.MACRO PRINT2 channel, pointer
	; Prepare X
	SetCIOChannelOffset channel

	LDA #ATARIOS_ICCOM::PutText
	STA AtariOS::ICCOM,X
	LDA pointer
	STA AtariOS::ICBA,X
	LDA pointer + 1
	STA AtariOS::ICBA + 1,X
	LDA #.LOBYTE(ICBL_TXT_MAXLENGTH)
	STA AtariOS::ICBL,X
	LDA #.HIBYTE(ICBL_TXT_MAXLENGTH)
	STA AtariOS::ICBL + 1,X
	JSR AtariOS::CIOV
.ENDMACRO



; *************************************************************************************************
; ??? - Outputs a single byte, perhaps?
; Parameters:
;   channel         Byte, The channel to use.
;   value           Byte, The value to put.
.MACRO PUT channel, value
	; Prepare X
	SetCIOChannelOffset channel

	LDA #ATARIOS_ICCOM::PutBinary
	STA AtariOS::ICCOM,X
	LDA #0
	STA AtariOS::ICBL,X
	STA AtariOS::ICBL + 1,X
	LDA #value
	JSR AtariOS::CIOV
.ENDMACRO



; *************************************************************************************************
; Does a binary put using an immediate length and immediate buffer address.
; Parameters:
;   channel         Byte, The channel to use.
;   length          Word, The length to use.
;   buffer          Address, The address of the buffer to use.
.MACRO BPUT channel, length, buffer
	; Prepare X
	SetCIOChannelOffset channel

	LDA #ATARIOS_ICCOM::PutBinary
	STA AtariOS::ICCOM,X
	LDA #.LOBYTE(length)
	STA AtariOS::ICBL,X
	LDA #.HIBYTE(length)
	STA AtariOS::ICBL + 1,X
	LDA #.LOBYTE(buffer)
	STA AtariOS::ICBA,X
	LDA #.HIBYTE(buffer)
	STA AtariOS::ICBA + 1,X
	JSR AtariOS::CIOV
.ENDMACRO



; *************************************************************************************************
; Does a binary put using a length variable and immediate buffer address.
; Parameters:
;   channel         Byte, The channel to use.
;   length          Address, The address to a word to use for the length.
;   buffer          Address, The address of the buffer to use.
.MACRO BPUT2 channel, length, buffer
	; Prepare X
	SetCIOChannelOffset channel

	LDA #ATARIOS_ICCOM::PutBinary
	STA AtariOS::ICCOM,X
	LDA length
	STA AtariOS::ICBL,X
	LDA length + 1
	STA AtariOS::ICBL + 1,X
	LDA #.LOBYTE(buffer)
	STA AtariOS::ICBA,X
	LDA #.HIBYTE(buffer)
	STA AtariOS::ICBA + 1,X
	JSR AtariOS::CIOV
.ENDMACRO



; *************************************************************************************************
; Does a binary put using a length variable and a buffer pointer variable.
; Parameters:
;   channel         Byte, The channel to use.
;   length          Address, The address to a word to use for the length.
;   buffer          Address, Address of a pointer to a word with the address of the buffer to use.
.MACRO BPUT3 channel, length, buffer
	; Prepare X
	SetCIOChannelOffset channel

	LDA #ATARIOS_ICCOM::PutBinary
	STA AtariOS::ICCOM,X
	LDA length
	STA AtariOS::ICBL,X
	LDA length + 1
	STA AtariOS::ICBL + 1,X
	LDA buffer
	STA AtariOS::ICBA,X
	LDA buffer + 1
	STA AtariOS::ICBA + 1,X
	JSR AtariOS::CIOV
.ENDMACRO



; *************************************************************************************************
; Does a binary get.
; Parameters:
;   channel         Byte, The channel to use.
;   length          Word, The length to use.
;   buffer          Address, The address of the buffer to use.
.MACRO BGET channel, length, buffer
	; Prepare X
	SetCIOChannelOffset channel

	LDA #ATARIOS_ICCOM::GetBinary
	STA AtariOS::ICCOM,X
	LDA #.LOBYTE(length)
	STA AtariOS::ICBL,X
	LDA #.HIBYTE(length)
	STA AtariOS::ICBL + 1,X
	LDA #.LOBYTE(buffer)
	STA AtariOS::ICBA,X
	LDA #.HIBYTE(buffer)
	STA AtariOS::ICBA + 1,X
	JSR AtariOS::CIOV
.ENDMACRO



; *************************************************************************************************
; Inputs text content.
; Parameters:
;   channel         Byte, The channel to use.
;   buffer          Address, The address of the buffer to use.
.MACRO INPUT channel, buffer
	; Prepare X
	SetCIOChannelOffset channel

	LDA #ATARIOS_ICCOM::GetText
 	STA AtariOS::ICCOM,X
 	LDA #.LOBYTE(buffer)
 	STA AtariOS::ICBA,X
 	LDA #.HIBYTE(buffer)
 	STA AtariOS::ICBA + 1,X
 	LDA #.LOBYTE(ICBL_TXT_MAXLENGTH)
 	STA AtariOS::ICBL,X
 	LDA #.HIBYTE(ICBL_TXT_MAXLENGTH)
 	STA AtariOS::ICBL + 1,X
  JSR AtariOS::CIOV
.ENDMACRO

.ENDIF
