; 6502 high speed SIO code
; License: GPL-2, see LICENSE
; This Version (c) 2022 Mark Ormston <memso@memso.net>
; Original copyright (c) 2003-2009 by Matthias Reichl <hias@horus.com>
;
; Changes:
; 2003-03-27 Matthias Reichl
;   - converted to Atasm source code format
;   - support for 8 (instead of 4) drives (*)
;   - use AtariOS::SETVBV instead of setting AtariOS::CDTMV1 directly (*)
;   - return 1 instead of 0 on success (*)
;   - added relocator table (*)
;   (*) thanks to Erhard Puetz for these fixes!
;
; 2004-01-19 Matthias Reichl
;   - removed relocator table for use with MyPicoDos
;
; 2004-01-22 Matthias Reichl
;   - changed all labels to local labels
;
; 2004-10-03 Matthias Reichl
;   - code merge with AtariSIO
;   - relocator table is generated if RELOCTABLE is defined
;   - moved definition of SIOADR out of this file
;
; 2005-08-15 Matthias Reichl
;   - added support for 1050 Turbo and XF551
;
; 2005-09-19 Matthias Reichl
;   - fixed Happy/Speedy bug introduced in last version
;   - code cleanup
;   - hispeed routine now honors value of AtariOS::ZP_SOUNDR
;
; 2006-05-01 Matthias Reichl
;   - fixed sporadic errors 139 with APE
;   - support for Happy 810 warp speed
;
; 2007-09-17 Matthias Reichl
;   - rewrote almost all old code, moved ZP memory addresses
;     to fix TurboDos issues
;
; 2007-10-07 Matthias Reichl
;   - fixed timeout handling, CLRTIM1 is now called after
;     receiving the whole data frame
;
; 2008-06-26 Matthias Reichl
;   - always configure happy drives to use fast writes
;     thanks to ijor for pointing out that otherwise the happy
;     might corrupt data when writing to the disk!
;
; 2008-09-05 Matthias Reichl
;   - added L_ABSxx labels to make the code relocatable again
;
; 2009-02-10 Matthias Reichl
;   - inlined checksum calculation into GETBLK to save
;     a few cycles
;   - added optional NMI patch to increase maximum speed
;     up to 110kbit/sec (pokey divisor 1)
;
; 2009-02-12 Matthias Reichl
;   - fix endless loop if device sends a speed byte of 0
;
; 2009-03-10 Matthias Reichl
;   - add support for pokey divisor 0
;   - implemented fast NMI code to solve overruns at divisor 0
;   - move GETBYTE into read data block loop to speed up the code
;   - implemented fast VBI code to make divisor 1 work reliably
;     without having to patch the OS
;
; 2009-05-01 Matthias Reichl
;   - speedup determining the highspeed mode (reduces time when
;     accessing a stock or non-existant drive for the first time)
;   - updated relocator table to include recent code changes
;
; 2010-08-14 Matthias Reichl
;   - optimized fast VBI code, NMI patch is now obsolete (and removed)

.INCLUDE "../Include/HISIO.inc"
.INCLUDE "../Include/AtariDOS.inc"

; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; START must be defined when assembling this file

; !!! Making it work. This adds a DOS header for the main code
.ORG START - 6
AtariDOS_BinaryStart START, MYEND - START

; *************************************************************************************************
; Extra if BEWEPATCH is defined
.IFDEF BEWEPATCH
	STDSIO:
		JMP AtariOS::SIOV

		LDA AtariOS::DDEVIC
		CMP #$31
		BNE STDSIO
		LDA AtariOS::DUNIT
		BEQ STDSIO
		CMP #5
		BCS STDSIO
.ENDIF



; *************************************************************************************************
; Include inner source
.INCLUDE "Detect.asm"
.INCLUDE "MergeHISIO.asm"



; *************************************************************************************************
; Various values

HIBUF:
	.RES 4, 0

SIOSPEED:
	.BYTE 0

SPEEDTB:
	.RES 8, 0



; *************************************************************************************************
; Set up for relocatable version
.IFDEF RELOCTABLE
	; Absolute addresses from hisiodet:
	; ABS21 must be WITHOUT + 1!
	RELOC:
		.WORD L_ABS01 + 1, L_ABS02 + 1, L_ABS03 + 1, L_ABS04 + 1, L_ABS05 + 1
		.WORD L_ABS06 + 1, L_ABS07 + 1, L_ABS08 + 1, L_ABS09 + 1, L_ABS10 + 1
		.WORD L_ABS11 + 1

		.IFNDEF USONLY
			.WORD L_ABS12A + 1, L_ABS12 + 1, L_ABS13 + 1, L_ABS14 + 1, L_ABS15 + 1, L_ABS16 + 1
		.ENDIF

		.WORD L_ABS17 + 1

		.IFNDEF USONLY
			.WORD L_ABS18 + 1
		.ENDIF

		.WORD L_ABS19 + 1, L_ABS20 + 1
		.WORD L_ABS21

		; Absolute addresses from hisiocode:
		; ABS67 must be WITHOUT  + 1!
		.WORD L_ABS50 + 1, L_ABS51 + 1, L_ABS52 + 1, L_ABS53 + 1, L_ABS54 + 1
		.WORD L_ABS55 + 1, L_ABS56 + 1, L_ABS57 + 1, L_ABS58 + 1, L_ABS59 + 1
		.WORD L_ABS60 + 1, L_ABS61 + 1, L_ABS62 + 1, L_ABS63 + 1

		.IFNDEF NOCKSUM
			.WORD L_ABS64 + 1
		.ENDIF
		.WORD L_ABS68 + 1, L_ABS69 + 1
		.WORD L_ABS70 + 1, L_ABS71 + 1, L_ABS72 + 1, L_ABS73 + 1, L_ABS74 + 1

		.IFDEF FASTVBI
			; ABSF08 must be WITHOUT  + 1!
			.WORD L_ABSF01 + 1, L_ABSF02 + 1, L_ABSF03 + 1, L_ABSF04 + 1
			.WORD L_ABSF05 + 1, L_ABSF06 + 1, L_ABSF07 + 1, L_ABSF08
		.ELSE
			.WORD L_ABS65 + 1, L_ABS66 + 1, L_ABS67
		.ENDIF

	RLEN:
		.WORD * - RELOC
.ENDIF

MYEND = *
