; highspeed SIO routine, status checks
; License: GPL-2, see LICENSE
; This Version (c) 2022 Mark Ormston <memso@memso.net>
; Original copyright (c) 2003-2009 by Matthias Reichl <hias@horus.com>

; *************************************************************************************************
; Check status.
; Incoming: ???
; Outgoing:
;   Assume all registers are lost. If a NAK or similar error occurred, see L_ERR.
L_CHKST:
	.IFDEF FASTVBI
		LDY #T2TIMEOUT
		LDX #1
	L_ABS68:
		JSR L_SMYTOUT

	.ELSE
		LDY #T2TIMEOUT
	L_ABS68:
		JSR L_STIMOUT
	.ENDIF

	; Set to receive IRQ
	LDA #MSKRECV
	STA POKEY_W::IRQEN
L_ABS69:
	; Read the acknowledgement
	JSR L_RDACK
	; Catch ERROR and handle it like NAK
	BNE L_ERRNAK

	; Fall through ------>
; End L_CHKST



; *************************************************************************************************
; A local RTS.
L_RDOK:
	RTS
; End L_RDOK



; *************************************************************************************************
; Set command timeout and wait for command complete.
; Incoming:
;   AtariOS::DTIMLO Byte, Specifies the time limit in seconds to use.
; Outgoing:
;   All registers are lost. If a NAK or similar error occurred, see L_ERR.
L_WAITCPL:
	; ===============================================================================================
	; Get the time limit
	.IFDEF FIXTOUT
		; Use a fixed time limit if specified
		LDX #>(FIXTOUT)
		LDY #<(FIXTOUT)
	.ELSE
		; Get the time limit and multiply by 64 for a rough estimate of the number of
		; frames for the time out.
		LDA AtariOS::DTIMLO
		ROR 
		ROR 
		TAY 
		AND #$3F
		TAX 
		TYA 
		ROR 
		AND #$C0
		TAY
	.ENDIF


	; ===============================================================================================
	; Set the time out time
	.IFDEF FASTVBI
		; hi value must be incremented
		INX
	L_ABS70:
		JSR L_SMYTOUT
	.ELSE
	L_ABS70:
		JSR L_STIMOU2
	.ENDIF

	; Fall through ------>
; End L_WAITCPL



; *************************************************************************************************
; Receive ACK/NAK, COMPLETE/ERROR
; Outgoing:
;   Zero set on success.
;   Assume all registers are lost. If a NAK or similar error occurred, see L_ERR.
L_RDACK:
	; Set pokey to receive async mode
	LDA #SKRECV
	STA POKEY_W::SKCTL
	; Reset serial/keyboard data (POKEY_R::SKSTAT)
	STA POKEY_W::SKREST

	; Clear command line
	LDA #PIA_CTL::SIO_Output_NegateLine | PIA_CTL::AccessIORegisters
	STA PIA::PBCTL

	; Read a single byte
	SEC
L_ABS71:
	JSR L_GETBYT

	.IFDEF DIAGSIO
		; Store the byte in the diagnostic data if used
		STA L_DIAGBT
	.ENDIF

	; ACK - Exit if all good
	CMP #'A'
	BEQ L_RDOK

	; COMPLETE - Exit if all good
	CMP #'C'
	BEQ L_RDOK

	; Command ERROR
	CMP #'E'
	BEQ L_RDERR

	; Fall through ------>
; End L_RDACK



; *************************************************************************************************
; Set the error status to NAK or misc error (ATARIOS_CIO_STATUS::DeviceNotACK).
; Outgoing: See L_ERR
L_ERRNAK:
	; NAK or any other error -> set code $8B
	LDA #ATARIOS_CIO_STATUS::DeviceNotACK
	IgnoreNextWord

	; Fall through ------>
; End L_ERRNAK



; *************************************************************************************************
; Set the error status to a framing error (ATARIOS_CIO_STATUS::FrameError).
; Outgoing: See L_ERR
L_ERRFRM:
	; framing error
	LDA #ATARIOS_CIO_STATUS::FrameError
	IgnoreNextWord

	; Fall through ------>
; End L_ERRFRM



; *************************************************************************************************
; Set the error status to a data input overrun (ATARIOS_CIO_STATUS::Overrun).
; Outgoing: See L_ERR
L_ERROVR:
	; data input overrun
	LDA #ATARIOS_CIO_STATUS::Overrun
	IgnoreNextWord

	; Fall through ------>
; End L_ERROVR



; *************************************************************************************************
; Set the error status to a time out error (ATARIOS_CIO_STATUS::Timeout).
; Outgoing: See L_ERR
L_ERRTO:
	; Timeout error
	LDA #ATARIOS_CIO_STATUS::Timeout

	; Fall through ------>
; End L_ERRTO



; *************************************************************************************************
; General error routine, code in A
; Incoming:
;   A               Byte, The error status to use. See ERR* constants.
; Outgoing:
;   AtariOS::ZP_STATUS is set to the error status.
;   The error is tracked in the diagnostic data if DIAGSIO is used.
;   This does *NOT* return, but instead jumps to L_SIOEND.
;   Stack is reset to AtariOS::STACKP.
;   All other registers are lost.
L_ERR:
	; Set the status
	STA AtariOS::ZP_STATUS

	.IFDEF DIAGSIO
		; X register contains last received character
		STX L_DIAGX
	
		; store status/error
		LDX L_DIAGER
		STA L_DIAGE,X
	
		; store last received character
		LDA L_DIAGX
		STA L_DIAGB2,X
	
		; current byte count
		TYA
		STA L_DIAGC,X
	
		; phase/stage
		LDA L_DIAGST
		STA L_DIAGS,X
	
		; last received ack/nack byte
		LDA L_DIAGBT
		STA L_DIAGB,X
	
		INC L_DIAGER
	.ENDIF

	; Reset the stack pointer
	LDX AtariOS::STACKP
	TXS

L_ABS72:
	; we got an error, cleanup and try to restart
	JMP L_SIOEND
; End L_ERR



; *************************************************************************************************
; Sets AtariOS::ZP_STATUS to ATARIOS_CIO_STATUS::DeviceError.
; Outgoing:
;   A is lost (#ATARIOS_CIO_STATUS::DeviceError)
L_RDERR:
	; command error
	LDA #ATARIOS_CIO_STATUS::DeviceError
	STA AtariOS::ZP_STATUS
	RTS
; End L_RDERR
