; highspeed SIO routine, main code
; License: GPL-2, see LICENSE
; This Version (c) 2022 Mark Ormston <memso@memso.net>
; Original copyright (c) 2003-2009 by Matthias Reichl <hias@horus.com>

; *************************************************************************************************
; Standard high speed SIO handler.
; Incoming:
;   AtariOS::DUNIT           Byte, Probably best to just leave this at 1.
;   AtariOS::DDEVIC          Byte, The device to use. eg, '1' for D1:
;   AtariOS::DCOMND          Byte, The SIO command to use.
;   AtariOS::DBUF            Address, The address of the read or write buffer, if needed.
;   AtariOS::DBYT            Word, The length of the read or write buffer, if needed.
;   AtariOS::DAUX1           Byte, The auxiliary value to use.
;   AtariOS::DAUX2           Byte, The auxiliary 2 value to use.
;   AtariOS::DSTATS          Byte, Flags for what to do. Set bit 7 for write, set bit 6 for read.
; Outgoing:
;   Not only has a considerable amount of time gone by, potentially several seconds, but:
;   All registers are lost.
DOHISIO:
	; ===============================================================================================
	; Diag SIO version
	.IFDEF DIAGSIO
		; Record current timestamp if specified
		.IFDEF DIAGTIM
		  LDA #0
		  STA ANTIC_W::NMIEN
		  LDA AtariOS::ZP_RTCLOK
		  STA L_TSTAMP
		  LDA AtariOS::ZP_RTCLOK + 1
		  STA L_TSTAMP + 1
		  LDA AtariOS::ZP_RTCLOK + 2
		  STA L_TSTAMP + 2
		  LDA ANTIC_R::VCOUNT
		  STA L_TSTAMP + 3
			LDA #$40
			STA ANTIC_W::NMIEN
		.ENDIF

		; Do not try detection
		LDA #$01


	; ===============================================================================================
	; Standard version
	.ELSE
		; Set the number of attempts to make
		.IFDEF DIAGNORET
			LDA #$01
		.ELSE
			LDA #$02
		.ENDIF
	.ENDIF


	; ===============================================================================================
	; Shared

	; Get the number of command frame retries to do
	LDX #DEFCRETRY

	; Fall through ------>
; End DOHISIO



; *************************************************************************************************
; Internal entry point for highspeed SIO. This makes changes to the incoming SIO command as
; necessary, as well as controlling the speed of communication.
; Incoming:
;   See DOHISIO.
;   A               Byte, The number of attempts to make. If this is 1, there is no speed fallback.
;   X               Byte, The number of command frame retries.
; Outgoing:
;   All registers are lost
DOHIDET:
	; ===============================================================================================
	; Prepare data

	; Store the parameters
	STA HISIO_DRETRY
	STX HISIO_CRETRY

	; Disable interrupts
	SEI

	; Set AtariOS::ZP_CRITIC to 1, because critical code is just starting!
	LDA #$01
	STA AtariOS::ZP_CRITIC

	; Set AtariOS::CDEVIC = AtariOS::DDEVIC + AtariOS::DUNIT - 1
	CLC
	LDA AtariOS::DDEVIC
	ADC AtariOS::DUNIT
	ADC #$FF
	STA AtariOS::CDEVIC


	; ===============================================================================================
	; Set up fast VBI if specified
	.IFDEF FASTVBI
	L_ABSF01:
		JSR L_CLRTIM1

		; Save old immediate VBI handler
		LDA AtariOS::VVBLKI
		PHA
		LDA AtariOS::VVBLKI + 1
		PHA

		; Install fast immediate VBI handler
		.IFDEF RELOCTABLE
		L_ABSF02:
			LDY L_VBIADR
		L_ABSF03:
			LDX L_VBIADR + 1
		.ELSE
			LDY #<(VBICODE)
			LDX #>(VBICODE)
		.ENDIF

		; Set the vertical blank immediate vector
	L_ABSF04:
		JSR L_WTVBI
		STY AtariOS::VVBLKI
		STX AtariOS::VVBLKI + 1

		; Updates if not ignoring the real-time clock
		.IFNDEF FASTVBI_NOCLOCK
			; Save mid AtariOS::ZP_RTCLOK byte for later adjustment of high byte
			LDA AtariOS::ZP_RTCLOK + 1
			PHA
		.ENDIF
	.ENDIF


	; ===============================================================================================
	; Prepare for the detection loop

	; Save the initial stack pointer
	TSX
	STX AtariOS::STACKP

L_ABS50:
	LDA SIOSPEED
	STA HISIO_SPEED

	; Fall through ------>
; End DOHIDET



; *************************************************************************************************
; Device retry loop
L_DRETLP:
	.IFDEF DIAGSIO
		LDA #0
		STA L_DIAGER
	.ENDIF

	; Get the passed-in auxiliary bytes
	LDA AtariOS::DAUX1
	STA AtariOS::CAUX1
	LDA AtariOS::DAUX2
	STA AtariOS::CAUX2

	; Get the command to use
	LDA AtariOS::DCOMND
	STA AtariOS::CCOMND

	; Get the speed specified
	LDX HISIO_SPEED

	.IFNDEF USONLY
		BPL L_NOTUR

		LDA AtariOS::CAUX2
		ORA #$80
		STA AtariOS::CAUX2
		BNE L_CRETLP

	L_NOTUR:
		; Happy warp speed?
		CPX #HAPFLG
		BEQ L_WARP

		; XF551?
		CPX #XFFLG
		BEQ L_XFCMD
	.ENDIF

	; check for happy 1050 speed
	CPX #$0A
	BNE L_CRETLP

	; the happy 1050 doesn't like the $48 happy command in highspeed...
	CMP #$48
	.IFDEF USONLY
		BNE L_CRETLP
		LDA #STDSPD
		STA HISIO_SPEED
	.ELSE
		BEQ L_DOSLOW
		BNE L_CRETLP
	.ENDIF
	; Fall through ------>
; L_DRETLP



; *************************************************************************************************
; Special additional functions if not US-only
.IFNDEF USONLY
	; =*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=
	; Prepares the XF551 as necessary before heading to the main command retry loop.
	; Incoming:
	;   A             Byte, The SIO command to use.
	L_XFCMD:
		; Set the high bit in the command for the high-speed version
		ORA #$80
		STA AtariOS::CCOMND

		; If the result is $A1 or $A2, use slow speed, otherwise proceed to the command retry loop
		CMP #$A1
		BEQ L_DOSLOW
		CMP #$A2
		BNE L_CRETLP

		; Fall through ------>
	; End L_XFCMD



	; =*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=
	; Forces the drive to slow down to the default speed. Used on fast drives with commands they don't
	; like handling at full speed. This sets A to STDSPD then jumps to L_CRETLP.
	L_DOSLOW:
		LDA #STDSPD
		STA HISIO_SPEED
		; Always taken
		BNE L_CRETLP
	; End L_DOSLOW



	; =*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=
	; Handles extras to set up a Happy drive's warp speed operations before heading to the main
	; command retry loop.
	; Incoming:
	;   A             Byte, The SIO command to use.
	L_WARP:
		; If the command is $50, $52, or $57, set bit 5 ($20) for warp speed, otherwise use the standard
		; speed
		CMP #$50
		BEQ L_WARP1
		CMP #$52
		BEQ L_WARP1
		CMP #$57
		BNE L_DOSLOW
	L_WARP1:
		ORA #$20
		STA AtariOS::CCOMND

		; Fall through ------>
	; End L_WARP
.ENDIF



; *************************************************************************************************
; Command retry loop.
; Incoming: See DOHISIO above.
L_CRETLP:
	; ===============================================================================================
	; Set up command speed
	.IFDEF DIAGSIO
		; Phase 00 : send command frame and wait for ACK
		LDA #0
		STA L_DIAGST
		STA L_DIAGBT
	.ENDIF

	; Get the speed to use for this command.
	LDA HISIO_SPEED

	; Switch to standard speed if the speed is a special speed.
	.IFNDEF USONLY
		CMP #MINSPECIALSPEED
		BCC L_DOSIO2
		LDA #STDSPD
L_DOSIO2:
	.ENDIF

	STA POKEY_W::AUDF3
	; Set command line
	LDA #$34
	STA PIA::PBCTL


	; ===============================================================================================
	; Prepare details for sending the command frame

	; Clear the status, high length, and POKEY_W::AUDF4
	LDA #$00
	STA AtariOS::ZP_STATUS
	STA AtariOS::ZP_BFENHI
	STA POKEY_W::AUDF4

	; Use the AtariOS::CDEVIC command buffer.
	LDA #<(AtariOS::CDEVIC)
	STA AtariOS::ZP_BUFRLO
	LDA #>(AtariOS::CDEVIC)
	STA AtariOS::ZP_BUFRHI

	; Set the buffer length to 4 since A is 2.
	ASL
	STA AtariOS::ZP_BFENLO

	; Set POKEY_W::AUDCTL to use the CPU's 1.79MHz clock and enable linking channel 3 + 4
	LDA #POKEY_AUDCTL::Link3And4 | POKEY_AUDCTL::Channel3Fast
	STA POKEY_W::AUDCTL

	; Set or disable disk loading sound
	LDX #$A0
	LDA AtariOS::ZP_SOUNDR
	BEQ L_NOSND
	LDX #$A8
L_NOSND:
	STX POKEY_W::AUDC4


	; ===============================================================================================
	; Send the command frame
L_ABS51:
	JSR L_SENDBLK


	; ===============================================================================================
	; Get the ACK/NAK byte

	.IFNDEF USONLY
		; If this is a 1050 Turbo, change to turbo speed for the ACK/NAK
		BIT HISIO_SPEED
		BPL L_NOTUR1 ; 1050 Turbo?
		LDA #TURSPD
		STA POKEY_W::AUDF3
	L_NOTUR1:
	.ENDIF

L_ABS52:
	; read command frame ACK/NAK
	JSR L_CHKST
L_ABS53:
	; got ACK, clear timer
	JSR L_CLRTIM1

	.IFDEF DIAGSIO
		; Phase 01 : send data frame (only if write operation)
		INC L_DIAGST
	.ENDIF

	.IFNDEF USONLY
		BIT HISIO_SPEED
		BVC L_NOXF	; XF551 / Happy Warp?

		LDA #XFSPD	; yes, set highspeed before DATA
		STA POKEY_W::AUDF3
	L_NOXF:
	.ENDIF

	; no more command-frame retries
	LDA #1
	STA HISIO_CRETRY


	; ===============================================================================================
	; Prepare for read/write operation

	; Copy the buffer address
	LDA AtariOS::DBUF
	STA AtariOS::ZP_BUFRLO
	LDA AtariOS::DBUF + 1
	STA AtariOS::ZP_BUFRHI

	; Copy the buffer length
	LDA AtariOS::DBYT
	STA AtariOS::ZP_BFENLO
	LDA AtariOS::DBYT + 1
	STA AtariOS::ZP_BFENHI

	; Skip sending if this isn't a send operation
	LDA AtariOS::DSTATS
	BPL L_NOSEND


	; ===============================================================================================
	; Write operation
L_ABS54:
	; transmit data frame
	JSR L_SENDBLK

	.IFDEF DIAGSIO
		; Phase 02 (write) : receive data frame ACK
		INC L_DIAGST
	.ENDIF

L_ABS55:
	; receive data frame ACK/NAK
	JSR L_CHKST


	; ===============================================================================================
	; Past receiving a command (or write operation if this is for writing)
L_NOSEND:
	.IFDEF DIAGSIO
		; Phase 02 (read) / 03 (write) : receive command complete
		INC L_DIAGST
	.ENDIF

L_ABS56:
	; Set command timeout and wait for command complete success
	JSR L_WAITCPL

	; Jump to the end if not a read operation
	BIT AtariOS::DSTATS
	BVC L_SIOEND

	.IFDEF DIAGSIO
		; Phase 03 (read) / 04 (write) : read data frame (only if read operation)
		INC L_DIAGST
	.ENDIF

L_ABS57:
	; Receive data block
	JSR L_GETBLK

	; Fall through ------>
; End L_CRETLP



; *************************************************************************************************
; Ends handling an SIO action.
L_SIOEND:
L_ABS58:
	; reset timers and pokey
	JSR L_CLEANUP

	; Done if the command was successful
	LDY AtariOS::ZP_STATUS
	BEQ L_ENDCMD

	; Retry sending command frame?
	DEC HISIO_CRETRY
	; Exit if already tried 13 times
	BEQ L_CHKDR
L_ABS59:
	; yes, we have some more retries
	JMP L_CRETLP

L_CHKDR:
	; End if there are not more command retries
	DEC HISIO_DRETRY
	BEQ L_ENDCMD

	; Don't fallback speed on command errors
	CPY #ATARIOS_CIO_STATUS::DeviceError
	BEQ L_GODR

	; Try it with standard speed this time
	LDA #STDSPD
	STA HISIO_SPEED

L_GODR:
	LDA #DEFCRETRY
	STA HISIO_CRETRY

L_ABS60:
	; Try it again
	JMP L_DRETLP
; END L_SIOEND



; *************************************************************************************************
; End an SIO command.
L_ENDCMD:
	; ===============================================================================================
	; Make corrections if the fast VBI was in use
	.IFDEF FASTVBI
		.IFNDEF FASTVBI_NOCLOCK
			; Store old AtariOS::ZP_RTCLOK mid byte in AtariOS::ZP_CHKSUM
			PLA
			STA AtariOS::ZP_CHKSUM
		.ENDIF

		PLA
		TAX
		PLA
		TAY

	L_ABSF05:
		JSR L_WTVBI
		STY AtariOS::VVBLKI
		STX AtariOS::VVBLKI + 1

		.IFNDEF FASTVBI_NOCLOCK
			; now check for mid-counter wrap-around:
			; if new value is smaller than old value, a wrap around occurred, so increment hi-counter
			LDA AtariOS::ZP_RTCLOK + 1
			CMP AtariOS::ZP_CHKSUM
			BCS L_NOWRAP
			INC AtariOS::ZP_RTCLOK
		L_NOWRAP:
		.ENDIF
	.ENDIF


	; ===============================================================================================
	; Final updates to end the operation

	; Clear critical code flag.
	LDA #0
	STA AtariOS::ZP_CRITIC

	; Restore POKEY_W::IRQEN
	LDA AtariOS::ZP_POKMSK
	STA POKEY_W::IRQEN

	; Get the final status and set it to 1 if clear
	LDY AtariOS::ZP_STATUS
	BNE L_ERRCMD
	INY
L_ERRCMD:
	STY AtariOS::DSTATS

	; Allow interrupts again
	CLI

	RTS
; End L_ENDCMD
