; highspeed SIO routine, receive byte/block code
; License: GPL-2, see LICENSE
; This Version (c) 2022 Mark Ormston <memso@memso.net>
; Original copyright (c) 2003-2010 by Matthias Reichl <hias@horus.com>

; NOTE: the code starting from "L_GETLP" up to the last "BNE L_GETLP"
; should not cross a page boundary, otherwise there's an increased
; chance for overruns at 126kbit/sec!

; *************************************************************************************************
; Gets a block of bytes.
; Note: POKEY is already set to receive mode (in rdquit), so we don't need to do the setup again
; Incoming:
;   AtariOS::ZP_BUFRLO/HI     Address, The buffer address to store the read data to.
;   AtariOS::ZP_BFENLO/HI     Word, The number of bytes to send.
; Outgoing:
;   All registers are lost.
;   AtariOS::ZP_CHKSUM, AtariOS::ZP_BUFRLO/HI, AtariOS::ZP_BFENLO/HI are all lost.
L_GETBLK:
	; Clear the checksum and Y
	LDY #0
	STY AtariOS::ZP_CHKSUM

	; Main loop
L_GETLP:
	; C=0 means receive block, C=1 receive single byte
	CLC

	; Fall through ------>
; End L_GETBLK



; *************************************************************************************************
; Get a byte OR gets a series of bytes if carry is clear.
; Incoming:
;   Carry                     If set, reading just a single byte. If clear, reading a buffer of bytes.
;  If reading a buffer of bytes:
;   Y                         Byte, The offset into AtariOS::ZP_BUFRLO/HI to use.
;   AtariOS::ZP_BUFRLO/HI     Address, The buffer address to store the read data to.
;   AtariOS::ZP_BFENLO/HI     Word, The number of bytes to send.
;   AtariOS::ZP_CHKSUM        Byte, The checksum for the data read. Not used if NOCKSUM is defined.
; Outgoing:
;  If reading just a byte:
;   A and X are the byte read.
;  If reading a buffer of bytes:
;   All registers are lost.
;   AtariOS::ZP_CHKSUM, AtariOS::ZP_BUFRLO/HI, AtariOS::ZP_BFENLO/HI are all lost.
L_GETBYT:
	; ===============================================================================================
	; Read a byte from the serial input

	; Wait for the receive status
	LDA #POKEY_IRQ::SerialInput
L_GETBY2:
	BIT POKEY_R::IRQST
	BPL L_ERRBRK ; Break key pressed?
	BNE L_GETBY2

	; Get a byte into X
	LDX POKEY_R::SERIN

	; Clear the receive status
	LDA #POKEY_IRQ::ClearSerialInput
	STA POKEY_W::IRQEN

	; Reenable the receive IRQ
	LDA #MSKRECV
	STA POKEY_W::IRQEN

	; Test for a data input overrun
	LDA #POKEY_SKSTAT::InputOverrun
	BIT POKEY_R::SKSTAT
L_ERRFR1:
	BPL L_ERRFRM ; framing error?
L_ERROV1:
	BEQ L_ERROVR ; overrun error?

	; Put the value read into A
	TXA

	; Done if just receiving a single byte (see L_GETLP above)
	; No commands in this function before this instruction changes carry
	BCS L_DORTS


	; ===============================================================================================
	; Buffer updates

	; Store the byte in the buffer
	STA (AtariOS::ZP_BUFRLO),Y

	; Update the checksum unless otherwise specified
	.IFNDEF NOCKSUM
		CLC
		ADC AtariOS::ZP_CHKSUM
		ADC #0
		STA AtariOS::ZP_CHKSUM
	.ENDIF

	; If Y looped around to 0, increase buffer page and decrease number of pages to output
	INY 
	BNE L_GET1
	INC AtariOS::ZP_BUFRHI
	DEC AtariOS::ZP_BFENHI
L_GET1:

	; Did Y reach the low length?
	CPY AtariOS::ZP_BFENLO
	BNE L_GETLP

	; Repeat if the high address hasn't been reached
	LDA AtariOS::ZP_BFENHI
	BNE L_GETLP


	; ===============================================================================================
	; Final updates

	; Get the next byte which should be the checksum
	SEC
L_ABS63:
	JSR L_GETBYT

	.IFDEF DIAGSIO
		; Store the checksum byte if diagnosing SIO
		STA L_DIAGBT
	.ENDIF

	.IFNDEF NOCKSUM
		; Did the checksum match?
		CMP AtariOS::ZP_CHKSUM
		BEQ L_DORTS

		; Error if the checksum didn't match
		LDA #ATARIOS_CIO_STATUS::ChecksumMismatch
	L_ABS64:
		JMP L_ERR
	.ENDIF

L_DORTS:
	RTS
; End L_GETBYT
