; highspeed SIO routine, send byte/block
; License: GPL-2, see LICENSE
; This Version (c) 2022 Mark Ormston <memso@memso.net>
; Original copyright (c) 2003-2009 by Matthias Reichl <hias@horus.com>

; *************************************************************************************************
; Send a byte.
; Note: This doesn't check for the break key here since it confuses several drives (eg. speedy 1050)
; Incoming:
;   A               Byte, The byte to send.
;   AtariOS::ZP_CHKSUM          Byte, The checksum.
; Outgoing:
;   A is the current AtariOS::ZP_CHKSUM.
;   X is the A value passed in.
;   Y is preserved.
;   AtariOS::ZP_CHKSUM is updated based on the new byte sent.
L_SENDBYT:
	TAX

	; Wait for the send status
	LDA #POKEY_IRQ::SerialOutput
L_WPUT:
	BIT POKEY_R::IRQST
	BNE L_WPUT

	; Clear the send status
	LDA #POKEY_IRQ::ClearSerialOutput
	STA POKEY_W::IRQEN

	; Reenable sending
	LDA #MSKSEND
	STA POKEY_W::IRQEN

	; Get the original value and output it
	TXA 
	STA POKEY_W::SEROUT

	; Fall through ------>
; End L_SENDBYT



; *************************************************************************************************
; Adds A to the checksum.
; Incoming:
;   A               Byte, The byte to add to the checksum.
; Outgoing:
;   A is the resulting AtariOS::ZP_CHKSUM.
;   AtariOS::ZP_CHKSUM is updated based on A sent.
L_ADDSUM:
	CLC 
	ADC AtariOS::ZP_CHKSUM
	ADC #$00
	STA AtariOS::ZP_CHKSUM
	RTS 
; End L_ADDSUM



; *************************************************************************************************
; Send a block of data.
; Incoming:
;   AtariOS::ZP_BUFRLO/HI     Address, The buffer address to send data from.
;   AtariOS::ZP_BFENLO/HI     Word, The number of bytes to send.
; Outgoing:
;   All registers are lost.
;   AtariOS::ZP_CHKSUM, AtariOS::ZP_BUFRLO/HI, AtariOS::ZP_BFENLO/HI are all lost.
L_SENDBLK:
	; Short delay before sending data
	; 1,281 cycles
	LDY #$00    ; 2
L_WT1:
	INY         ; 2
	BNE L_WT1   ; 3/2 [last loop] (so 5 per loop - 1)

	; Set pokey to transmit data mode
	LDA #SKSEND
	STA POKEY_W::SKCTL
	; Reset serial/keyboard data (POKEY_R::SKSTAT)
	STA POKEY_W::SKREST

	; Set send mode for the IRQ
	LDA #MSKSEND
	STA POKEY_W::IRQEN

	; Start transmission by sending first byte
	LDA (AtariOS::ZP_BUFRLO),Y
	STA AtariOS::ZP_CHKSUM
	STA POKEY_W::SEROUT
	INY

	; Y was incremented to 1, so this is always taken.
	BNE L_SND2
	
L_SND1:
	; Send bytes until Y wraps around
	LDA (AtariOS::ZP_BUFRLO),Y
L_ABS61:
	JSR L_SENDBYT
	INY 
	BNE L_SND2

	; Increment the buffer address and decrement the length
	INC AtariOS::ZP_BUFRHI
	DEC AtariOS::ZP_BFENHI

	; Wait a little after sending a full page, needed by some drives when sending more than 256 bytes
	; 159 cycles
	LDX #$E0   ; 2
L_WT2:
	INX        ; 2
	BNE L_WT2  ; 3/2 [last loop] (so 5 per loop - 1)

L_SND2:
	; Did Y hit the low byte of the buffer length?
	CPY AtariOS::ZP_BFENLO
	BNE L_SND1

	; Is the buffer done?
	LDA AtariOS::ZP_BFENHI
	BNE L_SND1

	; Send the checksum as the last byte
	LDA AtariOS::ZP_CHKSUM
L_ABS62:
	JSR L_SENDBYT

	; Wait until transmit complete
	LDA #POKEY_IRQ::Transmission
L_WTSND:
	BIT POKEY_R::IRQST
	BNE L_WTSND

	RTS
; End L_SENDBLK
