; Dump current OS to a file
; License: GPL-2, see LICENSE
; This Version (c) 2022 Mark Ormston <memso@memso.net>
; Original copyright (c) 2006 by Matthias Reichl

.INCLUDE "../Include/HISIO.inc"
.INCLUDE "../Include/CIO.inc"
.INCLUDE "../Include/AtariDOS.inc"

; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; Addresses

; Zero page vectors.
SRCVEC = $E0
DSTVEC = $E2

; Where the program starts.
START = $6000



; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; Code & Data

; !!! Making it work. This adds a DOS header for the main code
.ORG START - 6
AtariDOS_BinaryStart START, MYEND - START


; *************************************************************************************************
; The main copy code.
MAIN:
	; ===============================================================================================
	; Display the title and ask for a file name

	; Display the title
	BPUT 0, TTITL, TTIT

	; Get the file name
	INPUT 0, FNAM
	BPL FOK

	; Jump back to DOS
	JMP (AtariOS::ZP_DOSVEC)

FOK:
	; Attempt to open the file
	OPEN 1,8,0,FNAM
	BPL OPENOK

	; Display an error message and restart
	PRINT 0, TOPERR
	JMP MAIN

OPENOK:


	; ===============================================================================================
	; Save $C000 - $CFFF

	; Copy it to the buffer
	LDA #0
	STA SRCVEC
	LDA #$C0
	STA SRCVEC + 1
	LDX #$10
	JSR COPBLK

	; Safe the buffer to the disk
	BPUT 1,$1000,BUF
	BPL WOK1
	JMP WRERR
WOK1:


	; ===============================================================================================
	; Save selftest of original ROM

	; Disable interrupts
	SEI
	LDA #0
	STA ANTIC_W::NMIEN

	; Set $5000-$57FF to $FF, needed for dumping the old OS
	LDA #0
	STA SRCVEC
	LDA #$50
	STA SRCVEC + 1
	LDY #0
	LDX #8
	LDA #$FF
CLRLP:
	STA (SRCVEC),Y
	INY
	BNE CLRLP
	INC SRCVEC + 1
	DEX
	BNE CLRLP

	; Now (try to) enable selftest and copy it to buffer
	LDA PIA::PORTB
	PHA
	AND #$7F
	ORA #1
	STA PIA::PORTB

	; Copy the memory
	LDA #0
	STA SRCVEC
	LDA #$50
	STA SRCVEC + 1
	LDX #8
	JSR COPBLK

	; Restore PIA::PORTB
	PLA
	STA PIA::PORTB

	; Re-enable interrupts
	LDA #$40
	STA ANTIC_W::NMIEN
	CLI

	; Save the buffer
	BPUT 1,$0800,BUF
	BMI WRERR


	; ===============================================================================================
	; Save $D800 - $FFFF

	; Copy it to the buffer
	LDA #0
	STA SRCVEC
	LDA #$D8
	STA SRCVEC + 1
	LDX #$28
	JSR COPBLK

	; Save it to the file
	BPUT 1,$2800,BUF
	BMI WRERR


	; ===============================================================================================
	; Done successfully

	; Close the file
	CLOSE 1

	; Display a success message
	PRINT 0,TSUCC

	; Jump to wait
	JMP WAIT


	; ===============================================================================================
	; Errored
WRERR:
	; Close the file
	CLOSE 1

	; Display the error message
	PRINT 0,TWRERR

	; Fall through ------>
; End MAIN



; *************************************************************************************************
; Displays a message and waits. Either exits or restarts the program.
WAIT:
	; Display the message
	BPUT 0, TXITL, TXIT

	; Get a keypress
	JSR GETKEY

	; Exit if R not pressed, restart otherwise
	CMP #'R'
	BNE XIT
	JMP MAIN

XIT:
	JMP (AtariOS::ZP_DOSVEC)
; End WAIT



; *************************************************************************************************
; Waits for a keypress.
; Outgoing:
;   A               Byte, The key press from the user.
GETKEY:
	LDA AtariOS::KGETCHV + 1
	PHA
	LDA AtariOS::KGETCHV
	PHA
	RTS
; End GETKEY



; *************************************************************************************************
; Copies a block of memory to BUF.
; Incoming:
;   SRCVEC          Address, The source address to copy from.
;   X               Byte, The number of pages to copy.
; Outgoing:
;   All registers are lost.
;   SRCVEC and DSTVEC are lost.
COPBLK:
	LDA #<(BUF)
	STA DSTVEC
	LDA #>(BUF)
	STA DSTVEC + 1
	LDY #0
COPLP:
	LDA (SRCVEC),Y
	STA (DSTVEC),Y
	INY
	BNE COPLP
	INC SRCVEC + 1
	INC DSTVEC + 1
	DEX
	BNE COPLP
	RTS
; End COPBLK



; *************************************************************************************************
; Various Strings

TTIT:
	.BYTE ATASCII::EndOfLine, "OS dumper V1.02",ATASCII::EndOfLine
	.BYTE "(c) 2006-2009 Matthias Reichl",ATASCII::EndOfLine,ATASCII::EndOfLine
	.BYTE "filename >"
TTITL	= * -TTIT

TOPERR:
	.BYTE "error opening file!",ATASCII::EndOfLine
TWRERR:
	.BYTE "error writing file!",ATASCII::EndOfLine
TSUCC:
	.BYTE "successfully dumped OS!",ATASCII::EndOfLine

TXIT:
	.BYTE ATASCII::EndOfLine,"press 'R' to restart program,",ATASCII::EndOfLine
	.BYTE "any other key to exit",ATASCII::EndOfLine
TXITL	= * -TXIT

FNAM = *
MYEND = *

BUF	= * + $100



; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
; Store where the program starts for DOS

; !!! Making it work. Adds the start vector for DOS
.ORG AtariDOS::RUNAD
AtariDOS_BinaryBlock AtariDOS::RUNAD, 2
.WORD START
